<?php
/*
|--------------------------------------------------------------------------
| database.php 后台管理 数据库管理
|--------------------------------------------------------------------------
*/
namespace DeverApp\Manage;

use Dever;
use DeverApp\Manage\Lib\Input as Html;
use DeverApp\Manage\Lib\Log;

Dever::load('manage/auth.init');

class Database
{
	/**
     * project
     *
     * @var string
     */
	private $project;

	/**
     * table
     *
     * @var string
     */
	private $table;
	
	/**
     * menu
     *
     * @var string
     */
	private $menu;

	/**
     * menu_id
     *
     * @var string
     */
	private $menu_id;

	/**
     * id
     *
     * @var string
     */
	private $id;

	/**
     * config
     *
     * @var array
     */
	private $config;

	/**
     * search
     *
     * @var bool
     */
	private $search = false;

	/**
     * __construct
     * 
     * @return mixed
     */
	public function __construct()
	{
		$this->project 	= Dever::input('key');

		$this->table   	= Dever::input('table');

		$this->id 		= Dever::input('update_where_id') ? Dever::input('update_where_id') : Dever::input('where_id');
		
		$this->menu 	= Dever::input('menu');
		
		$this->menu_id 	= Dever::input('menu_id');
	}


	/**
     * url
     * 
     * @return array
     */
	private function url($key = false, $id = 0, $table = false)
	{
		$table = $table ? $table : $this->table;
		
		if(strpos($table, 'http://') !== false || strpos($table, 'javascript') !== false)
		{
			return $table;
		}

		if($key <= 3)
		{
			$where = Dever::preInput('option_');
			if($where)
			{
				if(isset($where['where_id']))
				{
					//unset($where['where_id']);
				}
				$table .= '&' . http_build_query($where);
			}
			
			$oper = Dever::preInput('oper_');
			if($oper)
			{
				$table .= '&' . http_build_query($oper);
			}
		}
		
		$param = 'key=' . $this->project . '&table=' . $table . '&menu=' . $this->menu . '&menu_id=' . $this->menu_id;
		$url = array
		(
			0 => Dever::url('project/database?key=' . $this->project),
			1 => Dever::url('project/database/list?' . $param),
			2 => Dever::url('project/database/update?' . $param),
			3 => Dever::url('project/database/update?' . $param . '&where_id=' . $id),
			4 => Dever::url('database.update_action'),
			5 => Dever::url(''),
			6 => Dever::url('database.delete_action?' . $param . '&where_id=' . $id),
			7 => Dever::url('project/database/stat?' . $param),
			8 => Dever::url('database.list_excel?' . $param),
			9 => Dever::url('database.stat_excel?' . $param),
			//特殊的自定义页面
			100 => Dever::url('list?' . $param),
		);

		if($key >= 10 && $key < 20)
		{
			return $url[1];
		}
		
		elseif($key >= 20 && $key < 30)
		{
			return $url[2];
		}

		if($key >= 0 && isset($url[$key]))
		{
			return $url[$key];
		}

		return $url;
	}

	/**
     * 获得refer 暂时无用了
     * 
     * @return string
     */
	private function refer($type = 0)
	{
		return '';

		switch($type)
		{
			case 0:
				$name = '数据库操作';
				break;
			case 1:
				$name = '数据列表';
				break;
			case 5:
				$name = '项目列表';
				break;
		}

		$name = '返回上一级';

		$url = $this->url($type);
		
		$refer = '-[<a href="'. $url .'">' . $name . '</a>]';

		return $refer;
	}

	/**
     * info 处理一些基本的信息，供mote模板使用
     * 
     * @return array
     */
	public function info()
	{
		$config = $this->config();

		$menu = isset($config['lang']) ? $config['lang'] : $this->project . '-' . $this->table;

		$display = '';
		
		# 新增权限
		$insert_state = Dever::load('manage/auth.oper', 4);
		
		if((isset($config['manage']['insert']) && $config['manage']['insert'] == false) || !isset($config['manage']['insert']) || $insert_state == false)
		{
			$display = 'display:none';
		}

		$info = array
		(
			'action' 			=> $this->url(4),
			'list' 				=> $this->url(1),
			'mul' 				=> $this->url(1),
			//'add' 				=> 'location.href=\'' . $this->url(2) . '\'',
			'add' 				=> $this->url(2),
			'add_state'			=> $display,

			'project' 			=> $this->project,
			'table' 			=> $this->table,
			'id' 				=> $this->id,
			'main' 				=> '数据管理' . $this->refer(5),

			'list_header' 		=> $menu,
			'list_desc' 		=> '数据列表',
			'project_name'		=> Dever::load('manage/project.name', $this->project),
			'update_header' 	=> $menu,
			'update_desc' 		=> '更新数据',
			
			'updateMul'			=> (!$this->id && isset($config['manage']['updateMul']) && $config['manage']['updateMul']) ? '增加一条' : false,
			'updateDesc'		=> isset($config['manage']['desc']) ? $config['manage']['desc'] : '',
			'listDesc'			=> isset($config['desc']) ? $config['desc'] : '',
			'struct'			=> isset($config['manage']['struct']) ? $config['manage']['struct'] : '',
			'stat_desc' 		=> '主动统计',
		);
		
		$info['parent'] = Dever::input('oper_parent');
		if($info['parent'])
		{
			$oper = Dever::preInput('oper_');
			if($oper)
			{
				$info['parent'] .= '&' . str_replace('oper_', '', http_build_query($oper));
			}
			//$info['parent'] = 'location.href=\'' . $this->url(Dever::input('oper_index', 11), 0, $info['parent']) . '\'';
			$info['parent'] = $this->url(Dever::input('oper_index', 11), 0, $info['parent']);
			$info['parent_state'] = '';
		}
		else
		{
			$info['parent_state'] = 'display:none';
		}
		
		$info['save'] = Dever::input('oper_save_jump');
		if($info['save'])
		{
			$info['list'] = $this->url(Dever::input('oper_index', 11), 0, $info['save']);
		}
		
		$search = Dever::preInput('search_');
		if($search)
		{
			$info['mul'] .= '&' . http_build_query($search);
		}

		if($this->id > 0)
		{
			$info['update_header'] = $menu . '-更新数据' . $this->refer(1);
		}

		return $info;
	}

	/**
     * 获取表名
     * 
     * @return array
     */
	public function getTableName($param)
	{
		if($param[1] == 'other') return '无数据表';

		$path = Dever::load('manage/project.path', $param[0]);

		$config = $this->config($param[0], $path, $param[1]);

		return isset($config['lang']) ? $config['lang'] : $param[0];
	}

	/**
     * config
     * 
     * @return array
     */
	public function config($project = false, $path = false, $table = false)
	{
		if(!$project)
		{
			$project 	= $this->project;
			$table 		= $this->table;
		}
		if(!$project)
		{
			return array();
		}
		Dever::$global['manage']['project'] = $project;

		$config = Dever::load('manage/project.config', $project);
		if(!is_array($config))
		{
			Dever::abert($project . ' error');
		}
		$project = $config['lib'];

		if(!$path)
		{
			if(isset($config['base']) && isset($config['rel']))
	        {
	            $path = DEVER_INCLUDE_PATH . $config['rel'];
	        }
	        else
	        {
	            $path = $config['path'];
	        }
		}

		if($project && $path && empty($this->config[$path]))
		{
			$this->config[$path] = array();
			$dir = $path . 'database/';
			if(is_dir($dir))
			{
				$database = scandir($dir);

				foreach($database as $k => $v)
				{
					if(strpos($v, '.php') !== false)
					{
						$k = str_replace('.php', '', $v);
						$this->config[$path][$k] = $this->struct($dir . $v, $project);
					}
				}
			}
		}

		return ($table && isset($this->config[$path][$table])) ? $this->config[$path][$table] : $this->config[$path];
	}
	
	/**
     * 对database配置文件进行解析
     * 
     * @return array
     */
    private function struct($file, $project = false)
    {
		$config = include($file);
		
		if(isset($config['struct']) && is_array($config['struct']['id']))
		{
			$config['update'] = false;
			
			$config['manage']['struct'] = '<table border=1 width=100% height=100%>';
			
			foreach($config['struct'] as $k => $v)
			{
				if(isset($v['search']))
				{
					/*
					if(strpos($v['search'], 'order') !== false)
					{
						$config['manage']['search']['order'][$k] = $v['name'];
					}
					*/

					if(strpos($v['search'], 'time') !== false)
					{
						$config['manage']['search']['time'][$k] = $v['name'];
					}

					if(strpos($v['search'], 'date') !== false)
					{
						$config['manage']['search']['date'][$k] = $v['name'];
					}

					if(strpos($v['search'], 'fulltext') !== false)
					{
						$config['manage']['search']['fulltext'][$k] = $v['name'];

						isset($v['match']) && $config['request']['list']['option'][$k] = array('option', 'like');
					}

					if(isset($v['option']) && strpos($v['search'], 'select') !== false)
					{
						$config['manage']['search']['select'][$k]['option'] = $v['option'];
						$config['manage']['search']['select'][$k]['lang'] = $v['name'];
						if(isset($v['default']))
						{
							$config['manage']['search']['select'][$k]['default'] = $v['default'];
						}
					}
					
					if(isset($v['option']) && strpos($v['search'], 'group') !== false)
					{
						$config['manage']['search']['group'][$k]['option'] = $v['option'];
						$config['manage']['search']['group'][$k]['lang'] = $v['name'];
						if(isset($v['default']))
						{
							$config['manage']['search']['group'][$k]['default'] = $v['default'];
						}
					}

					if(isset($v['option']) && strpos($v['search'], 'mul') !== false)
					{
						$config['manage']['search']['mul'][$k]['option'] = $v['option'];
						$config['manage']['search']['mul'][$k]['lang'] = $v['name'];
						if(isset($v['default']))
						{
							$config['manage']['search']['mul'][$k]['default'] = $v['default'];
						}
					}
				}
				
				if(isset($v['table']) && !isset($config['end']))
				{
					$config['end']['insert'] = $v['value'];
				}

				if(isset($v['update']))
				{
					if(!isset($config['manage']['insert']))
					{
						$config['manage']['insert'] = true;
					}
					$config['update'] = true;
				}

				if(isset($v['type']))
				{
					$config['manage']['struct'] .= Dever::tbody(array($k, $v['type'], $v['name']));
				}

			}
			
			$config['manage']['struct'] .= '</table>';
			
			$config['manage']['_list'] = array
			(
				'col' => 'manage',
				'name' => '管理',
				'value' => array(),
			);
			
			if(!isset($config['manage']['edit']) && $config['update'] == true)
			{
				# 3和6
				$config['manage']['_list']['value'] = array(3 => '编辑');
			}
			if(isset($config['manage']['list'][0]) && is_array($config['manage']['list'][0]))
			{
				$config['manage']['_list']['value'] += $config['manage']['list'][0];
				unset($config['manage']['list'][0]);
			}
			elseif(isset($config['manage']['list_button']))
			{
				$config['manage']['_list']['value'] += $config['manage']['list_button'];
			}
			
			if(!$config['manage']['_list']['value'])
			{
				unset($config['manage']['_list']);
			}
		}
		
		if(isset($config['top']))
		{
			if(is_array($config['top']))
			{
				$config['top']['key'] = $project . '-' . $config['top']['key'];
			}
			else
			{
				$config['top_key'] = $config['top'];
				$config['top'] = $project . '-' . $config['top'];
			}
		}
		
		return $config;
	}

	/**
     * 自定义列表页面
     * 
     * @return array
     */
	public function page($key = 'list')
	{
		$key = Dever::input('page_value', $key);
		$type = Dever::input('page_type', 1);
		$config = $this->config();

		$result = array();
		$result['s1'] = 'display:none';
		$result['s2'] = 'display:';
		$result['content'] = '';

		$key = 'page_' . $key;

		if(isset($config['manage'][$key]))
		{
			$result['content'] = Dever::view($config['manage'][$key], 'manage', $this->project);

			if($result['content'])
			{
				$result['s1'] = 'display:';
				$result['s2'] = 'display:none';

				# 添加底部
				$result['content'] .= Dever::view('foot', 'inc', 'manage');
			}
		}

		return $result;
	}

	/**
     * 自定义列表页面
     * 
     * @return array
     */
	public function page_list()
	{
		return $this->page('list');
	}

	/**
     * 自定义列表页面 只接管table部分，同时必须传值page_type=1
     * 
     * @return array
     */
	public function page_list_table()
	{
		$type = Dever::input('page_type', 2);
		if($type == 1)
		{
			return $this->page('list_table'); 
		}
		else
		{
			return $this->page('list_table_default'); 
		}
	}

	/**
     * 自定义列表页面
     * 
     * @return array
     */
	public function page_update()
	{
		return $this->page('update');
	}

	/**
     * 头部banner
     * 
     * @return array
     */
	public function banner()
	{
		if(!$this->project)
		{
			return '';
		}
		$config = $this->config();

		$result = '';

		if(isset($config['manage']['banner']))
		{
			foreach($config['manage']['banner'] as $k => $v)
			{
				$config['manage']['banner'][$k] = $this->value($v, 'yes');
			}
			if(empty($config['manage']['banner']['link']))
			{
				$config['manage']['banner']['link'] = '#';
				$config['manage']['banner']['target'] = '_self';
			}
			if(empty($config['manage']['banner']['target']))
			{
				$config['manage']['banner']['target'] = '_blank';
			}
			$result = '<a target="'.$config['manage']['banner']['target'].'" href="'.$config['manage']['banner']['link'].'"><img src="'.$config['manage']['banner']['img'].'"></a>
					<span>'.$config['manage']['banner']['name'].'</span>';
		}

		return $result;
	}

	/**
     * 列表页搜索
     * 
     * @return array
     */
	public function list_search_select(&$result, $search, $name, $lang, $option, $default = false)
	{
		$value = array();
		if(strpos($lang, '-'))
		{
			$temp = explode('-', $lang);
			$lang = $temp[0];
		}
		$value['name'] = $name;
		$value['lang'] = $lang;
		$value['option'] = $option;
		if(isset($search[$name]))
		{
			$value['value'] = $search[$name];
		}
		if($default)
		{
			//$value['default'] = $default;
		}
		$result .= Html::select($value, false) . '&nbsp;&nbsp;&nbsp;';
	}
	
	/**
     * 列表页搜索
     * 
     * @return array
     */
	public function list_search_group(&$result, $search, $name, $lang, $option, $default = false)
	{
		$value = array();
		$value['name'] = $name;
		$value['lang'] = $lang;
		$value['option'] = $option;
		if(isset($search[$name]))
		{
			$value['value'] = $search[$name];
		}
		if($default)
		{
			//$value['default'] = $default;
		}
		$result .= Html::group($value, false) . '&nbsp;&nbsp;&nbsp;';
	}

	/**
     * 列表页搜索
     * 
     * @return array
     */
	public function list_search_mul(&$result, $search, $name, $lang, $option, $default = false)
	{
		$value = array();
		$value['name'] = $name;
		$value['lang'] = $lang;
		$value['option'] = $option;
		if(isset($search[$name]))
		{
			$value['value'] = $search[$name];
		}
		if($default)
		{
			//$value['default'] = $default;
		}
		$result .= Html::mul($value, 'span') . '&nbsp;&nbsp;&nbsp;';
	}

	/**
     * 列表页搜索
     * 
     * @return array
     */
	public function list_search_time(&$result, $search, $prefix, $key, $lang, $type = 'time')
	{
		$value['name'] = $prefix . 'option_start_' . $key;
		$value['lang'] = '开始' . $lang;
		if(isset($search[$value['name']]))
		{
			$value['value'] = Dever::maketime($search[$value['name']]);
			Dever::setInput($value['name'], $value['value'], 'search_');
		}
		$result .= Html::$type($value) . '&nbsp;-&nbsp;';

		$value['name'] = $prefix . 'option_end_' . $key;
		$value['lang'] = '结束' . $lang;
		if(isset($search[$value['name']]))
		{
			$value['value'] = Dever::maketime($search[$value['name']]);
			Dever::setInput($value['name'], $value['value'], 'search_');
		}
		$result .= Html::$type($value) . '&nbsp;&nbsp;<br />';
	}
	
	/**
     * 列表页头部按钮
     * 
     * @return array
     */
	public function list_button()
	{
		# 新增权限
		if(Dever::load('manage/auth.oper', 2) == false)
		{
			return '';
		}
		
		$config = $this->config();
		
		$result = '';
		
		# 增加主动统计按钮
		if(isset($config['manage']['stat']))
		{
			$config['manage']['button']['数据统计'] = $this->url(7);
		}

		# 增加下载报表按钮
		if(isset($config['manage']['excel']))
		{
			$config['manage']['button']['导出数据'] = 'onclick="$(\'#form1\').attr(\'action\', \''.$this->url(8).'\');list_search($(this))"';
		}
		
		# 列表页按钮
		if(isset($config['manage']['button']))
		{
			$result .= $this->button_list($config['manage']['button']);
		}
		
		return $result;
	}

	/**
     * 更新页按钮
     * 
     * @return array
     */
	public function update_button()
	{
		# 新增权限
		if(Dever::load('manage/auth.oper', 2) == false)
		{
			return '';
		}

		$info = $this->info();

		$result = '<button type="button" class="btn btn-primary save-data"><i class="fa fa-save"></i> 提交保存</button>
      		<a href="'.$info['list'].'" class="btn btn-danger" onclicks="msg({status:1,msg:\'yes\'})">放弃保存</a>';
		
		$config = $this->config();
		
		if(isset($config['manage']['update_button']))
		{
			$result = '';
			foreach($config['manage']['update_button'] as $k => $v)
			{
				if($v == 'save-data')
				{
					$result .= '<button type="button" class="btn btn-primary save-data"><i class="fa fa-save"></i> '.$k.'</button>';
				}
				elseif($v == 'copy-data')
				{
					$result .= '<a href="javascript:;" class="btn btn-danger" onclick="set_update(\'#copy_num\', 1);set_update(\'#update_where_id\', 0);submit_action();">'.$k.'</a>';
				}
				elseif($v == 'copy-datas')
				{
					$result .= '<a href="#dever_modal" data-toggle="modal" class="btn btn-danger" onclick="copy_data($(this))">'.$k.'</a>';
				}
				else
				{
					$result .= '<a href="javascript:;" class="btn btn-danger" onclick="'.$v.'">'.$k.'</a>';
				}
			}
		}

		return $result;
	}
	
	private function button_list($button)
	{
		$result = '';
		foreach($button as $k => $v)
		{
			if(is_array($v))
			{
				//$result .= '&nbsp;&nbsp;<a href="javascript:;" class="btn btn-default" onclick="jump(\''.$this->url($v[0], false, $v[1]).'\')">'.$k.'</a>';

				$result .= '&nbsp;&nbsp;<a class="btn btn-default" href="'.$this->url($v[0], false, $v[1]).'">'.$k.'</a>';
			}
			elseif(strpos($v, 'modal|') !== false)
			{
				$t = explode('modal|', $v);
				$result .= '&nbsp;&nbsp;<a href="#dever_modal" data-toggle="modal"  class="btn btn-default" onclick="$(\'#dever_modal_body\').html(\''.$t[1].'\')">'.$k.'</a>';
			}
			elseif(strpos($v, '|') !== false)
			{
				$t = explode('|', $v);
				$result .= $t[0] . '&nbsp;&nbsp;<a href="javascript:;" class="btn btn-warning" '.$t[1].'>'.$k.'</a>';
			}
			elseif(strpos($v, 'onclick') !== false)
			{
				$result .= '&nbsp;&nbsp;<a href="javascript:;" class="btn btn-warning" '.$v.'>'.$k.'</a>';
			}
			elseif(strpos($v, 'http://') !== false)
			{
				//$result .= '&nbsp;&nbsp;<a href="javascript:;" class="btn btn-default" onclick="jump(\''.Dever::url($v).'\')">'.$k.'</a>';
				$result .= '&nbsp;&nbsp;<a class="btn btn-default" href="'.Dever::url($v).'">'.$k.'</a>';
			}
			else
			{
				$result .= '&nbsp;&nbsp;<a href="javascript:;" class="btn btn-danger" onclick="load(\''.Dever::url($v).'\')">'.$k.'</a>';
			}
		}
		
		return $result;
	}

	/**
     * 列表页搜索
     * 
     * @return array
     */
	public function list_search()
	{
		# 新增权限
		if(Dever::load('manage/auth.oper', 2) == false)
		{
			return '';
		}
		
		$config = $this->config();

		$result = '';

		# 搜索
		if(isset($config['struct']) && isset($config['manage']['search']))
		{
			$search = Dever::preInput('search_');

			$node = Html::node(array('class="am-u-sm-12" style="margin-top:10px;"', 'class="am-form-group am-input-group-sm"'));
			$btn = '<input type="hidden" value="2" name="excel" id="excel"><button class="btn btn-primary" onclick="$(\'#form1\').attr(\'action\', \''.$this->url(1).'\');list_search($(this))" type="button" style="height: 38px;margin-left:10px;">搜索</button>';

			//$result .= '<form id="f1" action="'.$this->url(1).'" method="post">';

			$prefix = 'search_';

			$result .= $node[0];

			if(isset($config['manage']['search']['time']))
			{
				foreach($config['manage']['search']['time'] as $k => $v)
				{
					$this->list_search_time($result, $search, $prefix, $k, $v);
				}
			}
			
			if(isset($config['manage']['search']['date']))
			{
				foreach($config['manage']['search']['date'] as $k => $v)
				{
					$this->list_search_time($result, $search, $prefix, $k, $v, 'date');
				}
			}

			if(isset($config['manage']['search']['select']))
			{
				foreach($config['manage']['search']['select'] as $k => $v)
				{
					$this->list_search_select($result, $search, $prefix . 'option_' . $k, $v['lang'], $this->option($v['option']), $v['default']);
				}
			}
			
			if(isset($config['manage']['search']['group']))
			{
				foreach($config['manage']['search']['group'] as $k => $v)
				{
					$this->list_search_group($result, $search, $prefix . 'option_' . $k, $v['lang'], $this->option($v['option']), $v['default']);
				}
			}

			if(isset($config['manage']['search']['mul']))
			{
				foreach($config['manage']['search']['mul'] as $k => $v)
				{
					$this->list_search_mul($result, $search, $prefix . 'option_' . $k, $v['lang'], $this->option($v['option']), $v['default']);
				}
			}

			/*排序加到列表表头中
			if(isset($config['manage']['search']['order']))
			{
				$this->list_search_select($result, $search, $prefix . 'order', '选择排序', $config['manage']['search']['order']);
				$this->list_search_select($result, $search, $prefix . 'desc', false, array('asc' => '正序','desc' => '倒序'));
			}
			*/

			if(isset($config['manage']['search']['fulltext']))
			{
				//$result .= '<br />';
				
				/*
				$this->list_search_select($result, $search, $prefix . 'fulltext', '选择检索项', $config['manage']['search']['fulltext']);
				
				$this->list_search_select($result, $search, $prefix . 'fulltext_type', '', array
				(
					1 => '精确匹配',
					2 => '模糊匹配',
					3 => '大于',
					4 => '大于等于',
					5 => '小于',
					6 => '小于等于',
				), 1);
				*/

				foreach($config['manage']['search']['fulltext'] as $k => $v)
				{
					$value = array();
					$value['name'] = $prefix.'fulltext_' . $k;
					$value['class'] = 'form-control';
					if(isset($search[$value['name']]))
					{
						$value['value'] = $search[$value['name']];
					}
					if(strpos($v, '-') !== false)
					{
						$t = explode('-', $v);
						$v = $t[0];
					}
					$value['placeholder'] = $v;
					$result .= Html::text($value, false) . '&nbsp;';
				}
			}

			# 列表页按钮
			if(isset($config['manage']['search_button']))
			{
				$btn .= $this->button_list($config['manage']['search_button']);
			}
		
			$result .= $btn . $node[1];

			//$result .= '</form>';
		}

		//print_r($result);die;

		return $result;
	}

	/**
     * 列表页表格头
     * 
     * @return array
     */
	public function list_thead()
	{
		# 新增权限
		if(Dever::load('manage/auth.oper', 1) == false)
		{
			return '';
		}
		
		$config = $this->config();

		$result = array();

		if(isset($config['struct']))
		{
			if(isset($config['manage']['list']) && $config['manage']['list'] == 'update')
			{
				# 验证头部权限 这里如果是多个选择权限的话，暂时无法实现。。。
				if(isset($config['top']) && is_string($config['top']))
				{
					$top = Dever::load('manage/auth.getTop', array($config['top']));
					if($top)
					{
						//Dever::setInput('update_' . $config['top'], $top['value']);
						//$common[$config['top']] = $top['value'];
						$url = $this->url(3, $top['value']);
					}
				}
				else
				{
					$url = $this->url(3, 1);
				}

				return Dever::location($url);
			}
			else
			{
				if(isset($config['manage']['mul']))
				{
					$result[] = '<th><input type="checkbox" class="checkbox-checkall" name="checkall" id="checkall" value="list"/></th>';
				}

				$admin = Dever::load('manage/auth.info');
				$data['admin_id'] = $admin['id'];
				$data['role_id'] = $admin['role'];

				foreach($config['struct'] as $k => $v)
				{
					if(isset($v['list']) && $v['list'])
					{
						# 处理权限
						if(isset($v['auth']))
						{
							$auth = $this->value($v['auth'], $data);
							if(!$auth)
							{
								continue;
							}
						}
						if(isset($v['list_name']))
						{
							$v['name'] = $v['list_name'];
						}
						else
						{
							$temp = Html::cue(array('name' => $v['name']));
							$v['name'] = $temp['name'];
						}
						
						if(isset($v['search']) && strpos($v['search'], 'order') !== false)
						{
							$link = Dever::url();
							$order = 'search_order_'.$k.'';
							if(strpos($link, $order) !== false)
							{
								$link = preg_replace('/[?|&]'.$order.'=(desc|asc)/i', '{order}', $link);
							}
							else
							{
								$link .= '{order}';
							}
							
							$search = Dever::input($order);
							
							$cancel = '';
							if($search)
							{
								$asc = $search;
								$cancel = '&nbsp;<a class="glyphicon glyphicon-remove-sign" style="color:black;" href="'.str_replace('{order}', '', $link).'" title="取消排序"></a>';
							}
							else
							{
								$asc = 'desc';
								if(!isset($v['order']) && $k != 'id')
								{
									$cancel = '&nbsp;<a class="glyphicon glyphicon-circle-arrow-down" style="color:black;" href="'.str_replace('{order}', '&' . $order . '=desc', $link).'" title="降序"></a>';
								}
							}
							
							if($asc == 'asc')
							{
								$asc = 'desc';
								$icon = 'down';
								$text = '降序';
							}
							else
							{
								$asc = 'asc';
								$icon = 'up';
								$text = '升序';
							}
							
							$link = str_replace('{order}', '&' . $order . '=' . $asc, $link);
							
							$result[] = '<th data-priority="1"><a href="'.$link.'" title="'.$text.'" style="color:black;">' . $v['name'] . '&nbsp;<span class="glyphicon glyphicon-circle-arrow-'.$icon.'"></span></a>'.$cancel.'</th>';
						}
						else
						{
							$result[] = '<th data-priority="1">' . $v['name'] . '</th>';
						}
					}
				}
				if((Dever::load('manage/auth.oper', 3) != false || Dever::load('manage/auth.oper', 5) != false) && isset($config['manage']['_list']['value']))
				{
					$result[] = '<th data-priority="1">'.$config['manage']['_list']['name'].'</th>';
				}
			}
		}

		return $result;
	}
	
	/**
     * 导出列表
     * 
     * @return array
     */
	public function list_excel()
	{
		Dever::$global['base']['excel'] = 1;
		$data = $this->list_tbody(true);
			 
		if($data)
		{
			$config = $this->config();

			$xls = new \PHPExcel();

			/*
			if(class_exists('\\PHPExcel'))
			{
				$xls = new \PHPExcel();
			}
			else
			{
				Dever::debug('Class \'PHPExcel\' not found');
				die;
			}
			*/
		
			$xls->getProperties()->setCreator("Maze Excel")
							 ->setLastModifiedBy("Maze Excel")
							 ->setTitle("Office 2007 XLSX Analysis Document")
							 ->setSubject("Office 2007 XLSX Analysis Document")
							 ->setDescription("Mazephp Excel")
							 ->setKeywords("Mazephp Excel")
							 ->setCategory("Mazephp Excel");
			
			$info = $this->info();
			
			$name =  $info['list_header'] . '的' . $info['list_desc'] . '-' . $info['project_name'];
			# 全局导出
			$start = 65;
			$act = $xls->setActiveSheetIndex(0);
			
			$act->setTitle($name);

			$i = 0;

			$admin = Dever::load('manage/auth.info');
			$data['admin_id'] = $admin['id'];
			$data['role_id'] = $admin['role'];

			foreach($config['struct'] as $k => $v)
			{
				if(isset($v['list']) && $v['list'])
				{
					# 处理权限
					if(isset($v['auth']))
					{
						$auth = $this->value($v['auth'], $data);
						if(!$auth)
						{
							continue;
						}
					}
					$num = 1;
					$cell = $this->_getChr($start, $i);
					$act->setCellValue($cell . '' . $num, $v['name']);
					$act->getColumnDimension($cell)->setWidth(20);
					$i++;
					
					foreach($data as $ki => $vi)
					{
						if(isset($vi[$k]))
						{
							$num += 1;
							
							$value = $vi[$k];
						
							if(is_string($v['list']))
							{
								$value = $this->value($v['list'], $data[$ki], $config['struct']);
							}
							elseif(isset($v['option']))
							{
								$v['option'] = $this->option($v['option']);
								if(isset($v['option'][$value]))
								{
									if(is_array($v['option'][$value]))
									{
										$value = $v['option'][$value]['name'];
									}
									else
									{
										$value = $v['option'][$value];
									}
								}
							}
						
							$act->setCellValue($cell . $num, $value);
						}
					}
				}
			}
			
			//$act->getStyle('A1:'.$k.'20')->getAlignment()->setVertical(PHPExcel_Style_Alignment::VERTICAL_CENTER);   // 垂直居中
			//$act->getStyle('A1:'.$k.'20')->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);// 水平居中
	    
			$filename = $name . '.xlsx';
			header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
			header('Content-Disposition: attachment;filename="'.$filename.'"');
			header('Cache-Control: max-age=0');

			$write = \PHPExcel_IOFactory::createWriter($xls, 'Excel2007');
			$write->save('php://output');
		}

		//print_r($data);die;
	}
	
	private function _getChr($start, $num)
    {
		$num = $start + $num;
		$max = floor($num/91);
		
		if($max > 0)
		{
			$num = $num - (91*$max) + $start;
			$num = chr($start) . chr($num);
		}
		else
		{
			$num = chr($num);
		}
		
		return $num;
	}

	/**
     * 列表页数据列表
     * 
     * @return array
     */
	public function list_tbody($excel = false)
	{
		# 新增权限
		if(Dever::load('manage/auth.oper', 1) == false)
		{
			return '';
		}
		
		$high = array();

		$search = Dever::load('manage/auth.oper', 2) ? Dever::preInput('search_') : false;
		
		$param = array();

		$parent = 0;

		if($search)
		{
			$fulltext = Dever::preInput('search_fulltext_');
			if($fulltext)
			{
				//Dever::setInput('option_' . $search['search_fulltext'], '%' . $search['search_fulltext_value'] . '%');
				$search['search_fulltext_type'] = 2;
				$fix = '';
				if($search['search_fulltext_type'] == 2)
				{
					$fix = '%';
				}
				
				foreach($fulltext as $k => $v)
				{
					if($v)
					{
						$k = str_replace('search_fulltext_', '', $k);
						$param['option_' . $k] = $fix . $v . $fix;
						unset($search[$k]);
						
						# 设定检索高亮
						$high[$k] = $v;
					}
					
				}

				if($param)
				{
					$param['search_type'] = $search['search_fulltext_type'];
				}
				$this->search = true;
				
			}
			
			$order = Dever::preInput('search_order_');
			if($order)
			{
				foreach($order as $k => $v)
				{
					$k = str_replace('search_order_', '', $k);
					$param['order'][0][$k] = $v;
					unset($search[$k]);
				}
			}

			foreach($search as $k => $v)
			{
				if($v)
				{
					if(strpos($k, 'option_') !== false)
					{
						//Dever::setInput(str_replace('search_', '', $k), $v);
						if(is_array($v))
						{
							if($v[0] == 0)
							{
								continue;
							}
							$v = str_replace(',0', '', implode(',', $v));
							$t = explode(',', $v);
							$parent = end($t);
							$v .= '%';
						}
						$param[str_replace('search_', '', $k)] = $v;
						$this->search = true;
					}
				}
			}

			//$search = Dever::preInput('option_');

			//print_r($search);die;
		}

		# 批量更新
		$mul = Dever::load('manage/auth.oper', 3) ? Dever::preInput('mul_') : false;

		if(Dever::input('method') == 'mul' && $mul && $mul['mul_type'] > 0)
		{
			if($mul['mul_type'] == 1 && empty($mul['mul_where_id']))
			{
				Dever::abert('您还未选择任何数据条目');
			}
			Dever::$global['base']['mul_type'] = $mul['mul_type'];

			unset($mul['mul_type']);
			foreach($mul as $k => $v)
			{
				if($v)
				{
					if(is_array($v))
					{
						$v = implode(',', $v);
					}
					$param[str_replace('mul_', '', $k)] = $v;
				}
			}

			if(isset($param))
			{
				$method = $this->project . '/' . $this->table . '-updatemul';
				$state = Dever::load($method, $param);

			}

			//print_r($param);die;
			Dever::abert('reload');
		}
		
		$config = $this->config();

		$result = array();

		$method = $this->project . '/' . $this->table . '-list';

		# 验证头部权限
		if(isset($config['top']) && is_string($config['top']))
		{
			$top = Dever::load('manage/auth.getTop', array($config['top']));
			if($top)
			{
				Dever::setInput('option_' . $config['top_key'], $top['value']);
			}
		}

		//print_r($param);
		$data = Dever::load($method, $param);
		
		if(isset($config['manage']['list_type']) && $config['manage']['list_type'] == 'parent')
		{
			$parent = array();
			$child = array();
			foreach($data as $k => $v)
			{
				if($v[$config['name'] . '_id'] <= 0)
				{
					$parent[$k] = $v;
				}
				else
				{
					$v['name'] = '|-----' . $v['name'];
					$child[$v[$config['name'] . '_id']][$k] = $v;
				}
			}

			if($parent)
			{
				$data = array();
				foreach($parent as $k => $v)
				{
					$data[$v['id']] = $v;
					if(isset($child[$v['id']]))
					{
						foreach($child[$v['id']] as $i => $j)
						{
							$data[$j['id']] = $j;
						}
					}
				}
			}
		}
		elseif(isset($config['manage']['list_type']) && $config['manage']['list_type'] == 'tree')
		{
			$data = $this->tree($data, $config, $parent);
		}

		if($excel == true)
		{
			return $data;
		}

		if(Dever::input('page_type') == 1)
		{
			Dever::$global['manage']['list_data'] = $data;

			return $data;
		}

		//print_r($data);die;
		//print_r(Helper::page("current"));die;

		if($data && isset($config['struct']))
		{
			foreach($data as $k => $v)
			{
				$admin = Dever::load('manage/auth.info');
				$data[$k]['admin_id'] = $admin['id'];
				$data[$k]['role_id'] = $admin['role'];

				$result[$k] = '<tr>';
				
				if(isset($config['manage']['mul']))
				{
					$result[$k] .= '<td><input type="checkbox" name="mul_where_id[]" class="checkbox-checkall-list" value="'.$v['id'].'"/></td>';
				}

				foreach($config['struct'] as $ki => $vi)
				{
					# 处理同步
					if(isset($vi['sync']) && strpos($ki, '-'))
					{
						$temp = explode('-', $ki);
						$load = Dever::load($temp[0] . '/' . $temp[1] . '-check', array('option_' . $vi['sync'][1] => $v[$vi['sync'][0]]));
						$v[$ki] = isset($load[$temp[2]]) ? $load[$temp[2]] : $vi['default'];
					}

					if(isset($vi['list']) && $vi['list'] && isset($v[$ki]))
					{
						# 处理权限
						if(isset($vi['auth']))
						{
							$auth = $this->value($vi['auth'], $data[$k]);
							if(!$auth)
							{
								continue;
							}
						}

						$show = $v[$ki];
						
						if(is_string($vi['list']))
						{
							$show = $this->value($vi['list'], $data[$k], $config['struct']);
						}
						if(isset($vi['key']) && $vi['update'] != 'editor')
						{
							$show = Dever::pic($show);
						}
						if(isset($vi['modal']))
						{
							$show = Html::modal($this->value($vi['modal'], $data[$k], $config['struct']), $show);
						}

						elseif(isset($vi['option']))
						{
							# 验证option是否是匿名函数
							$vi['option'] = $this->option($vi['option']);
							# 位运算
							if(isset($vi['bit']))
							{
								$ts = array();
								foreach($vi['option'] as $tm => $tv)
								{
									if(isset($vi['bit'][$tm]) && ($show & $tm) > 0)
									{
										$ts[] = $tv;
									}
								}
								$show = implode(',', $ts);
							}
							/*
							if((isset($vi['child_name']) || (isset($vi['show']) && is_string($vi['show']))))
							{
								if(isset($vi['option'][$show]) && is_array($vi['option'][$show]))
								{
									$show = $vi['option'][$show]['name'];
								}
								else
								{
									$show = isset($vi['option'][$show]) ? $vi['option'][$show] : $show;
								}
							}
							*/
							elseif((isset($vi['edit']) && $vi['edit']) || $ki == 'state')
							//elseif((isset($vi['edit']) && $vi['edit']))
							{
								$vi['name'] = 'up_col_' . $ki;
								$vi['value'] = $v[$ki];
								$m = 'select';
								if(isset($vi['update']) && $vi['update'] == 'group')
								{
									$m = 'group';
								}

								$show = Html::$m($vi, 'form-list', '', 'onchange="update($(this), \''.$v['id'].'\', \''.$this->project.'\', \''.$this->table.'\')"');
							}
							else
							{
								if(isset($vi['option'][$show]) && is_array($vi['option'][$show]))
								{
									$show = $vi['option'][$show]['name'];
								}
								else
								{
									$show = isset($vi['option'][$show]) ? $vi['option'][$show] : $show;
								}
							}
						}

						$text = $show;
						if($high && isset($high[$ki]))
						{
							$text = str_replace($high[$ki], '<font color="red">' . $high[$ki] . '</font>', $text);
						}

						if(!isset($vi['option']) && isset($vi['edit']) && $vi['edit'])
						{
							//$url = Dever::url($vi['edit'] . '?where_id=' . $v['id'] . '&col=' . $vi['col']. '&key=' . $this->project . '&table=' . $this->table);

							$url = $this->url(4);
							
							$vi['edit'] = $vi['edit'] == 'textarea' ? $vi['edit'] : 'text';

							$result[$k] .= '<td class="edit" data-col="'.$ki.'" data-url="' . $url . '" data-id="'.$v['id'].'" data-key="'.$this->project.'" data-table="'.$this->table.'" data-content="' . $show . '" data-type="'.$vi['edit'].'">' . $text . '</td>';
						}
						else
						{
							$result[$k] .= '<td>' . $text . '</td>';
						}
					}
				}

				if(isset($config['manage']['_list']['value']))
				{
					$result[$k] .= '<td>';

					foreach($config['manage']['_list']['value'] as $kj => $vj)
					{
						$pj = false;
						
						$h = '<span class="am-icon-pencil-square-o"></span>';
						$oper = true;
						if($kj == 3)
						{
							$oper = Dever::load('manage/auth.oper', 3);
						}
						elseif($kj == 6)
						{
							$oper = Dever::load('manage/auth.oper', 5);
							
							$h = '<span class="am-icon-trash-o"></span>';
						}
						
						if(is_array($vj) && isset($vj[1]))
						{
							if(isset($vj[2]))
							{
								$oper = $this->value($vj[2], $data[$k]);
							}
							if($oper && strpos($vj[1], 'modal|') === false)
							{
								$pj = $this->value($vj[1], $data[$k]);
								$vjn = $vj[0];
							}
						}
						else
						{
							$vjn = $vj;
						}

						if($oper == true)
						{
							if(isset($vj[1]) && strpos($vj[1], 'modal|') !== false)
							{
								$t = explode('|', $vj[1]);
								$result[$k] .= '<a href="#dever_modal" data-toggle="modal"  class="btn btn-info btn-rounded waves-effect waves-light m-b-5" onclick='.$t[1].' msg-send="'.$this->value($t[2], $data[$k]).'">' . $vj[0] . '</a>&nbsp;&nbsp;';
							}
							elseif(isset($vj[1]))
							{
								//print_r($vj);die;
								$result[$k] .= '<a href="' . $this->url($kj, $v['id'], $pj) . '" class="oper_' . $kj . '"><button type="button" class="btn btn-info btn-rounded waves-effect waves-light m-b-5">' . $h . $vjn . '</button></a>&nbsp;&nbsp;';
							}
							else
							{
								$result[$k] .= '<span style="margin-left:5px;margin-right:10px;">' . $vj[0] . '</span>';
							}
						}
					}
					$result[$k] .= '</td>';
				}

				$result[$k] .= '</tr>';
			}
		}

		return $result;
	}

	/**
     * 多级分类
     * 
     * @return array
     */
	private function tree($data, $config, $parent = 0)
	{
		$key = $config['name'] . '_id';
		$result = array();
		$this->treeData($data, $result, $key, $parent);

		return $result;
	}

	private function treeData($array, &$result, $key, $parent_id = 0, $level = 0)
	{
	    if(empty($array))
	    {
	    	return;
	    }

	    $level++;

	    foreach($array as $k => $v)
	    {
	        if(isset($v[$key . '_parent']) && $v[$key . '_parent'] == $parent_id)
	        {
	            $v['level'] = $level;
	            $v['name'] = $this->treeName($v['name'], $level);
	            $result[] = $v;
	            unset($array[$k]);
	            $this->treeData($array, $result, $key, $v['id'], $level);
	        }
	    }
	}

	private function treeName($name, $level = 1)
	{
		$string = '';

		if($level > 1)
		{
			$num = 4;
			$string = '-';
			//STR_PAD_LEFT
			$string = str_pad('|', $level*$num, $string);
		}

		return $string . $name;
	}
	
	/**
     * 列表页批量管理
     * 
     * @return array
     */
	public function list_mul()
	{
		# 新增权限:更新权限
		if(Dever::load('manage/auth.oper', 3) == false)
		{
			return '';
		}
		
		$config = $this->config();

		$result = '';

		if(isset($config['struct']) && isset($config['manage']['mul']))
		{
			$mul = Dever::preInput('mul_');

			$node = Html::node(array('class="am-u-sm-12" style="margin-top:10px;"', 'class="am-form-group am-input-group-sm"'));
			$btn = '<button class="btn btn-danger" onclick="$(\'#form1\').attr(\'action\', \''.$this->url(1).'\');list_mul($(this))" type="button" style="height: 38px;margin-left:10px;">批量更新</button>';

			$prefix = 'mul_';

			$result .= $node[0];
			
			$default = 0;
			
			$option = array
			(
				1 => '选中的数据',
			);

			# 只有当有搜索条件时才会出现
			if(Dever::load('manage/auth.oper', 2) && $this->search == true)
			{
				$option[2] = '所有数据';
				$default = 2;
			}

			$this->list_search_select($result, $mul, $prefix . 'type' . '" id="'.$prefix.'type', '请选择更新方式', $option, $default);
			
			foreach($config['struct'] as $k => $v)
			{
				if(isset($v['update']) && $v['update'] == 'radio' && isset($v['edit']) && (is_array($v['option']) || is_object($v['option'])))
				{
					$this->list_search_select($result, $mul, $prefix . 'set_' . $k, $v['name'], $this->option($v['option']), $v['default']);
				}
			}

			$result .= $btn . $node[1];
		}

		//print_r($result);die;

		return $result;
	}

	private function value($value, $data, $struct = array())
	{
		if(!$data)
		{
			return $value;
		}
		if(is_string($data))
		{
			$eval = '$value = ' . $value . ';';
			eval($eval);

			return $value;
		}
		if($value == 'table' && $struct)
		{
			$result = $this->struct_data($struct, $data);
			$value = Dever::table($result);
		}
		elseif(strpos($value, '{') !== false)
		{
			# 这里用到了/e，注意这里无论如何不能传入$_GET之类的用户参数 2015/7/21更新为callback方式
			//$eval = '$value = ' . preg_replace('/{(.*?)}/e', "\$data['$1']", $value) . ';';
			$func = function($r) use($data)
			{
				return $data[$r[1]];
			};
			$eval = '$value = ' . preg_replace_callback('/{(.*?)}/', $func, $value) . ';';
			eval($eval);
		}
		
		return $value;
							
	}

	/**
     * update
     * 
     * @return array
     */
	public function update()
	{	
		$config = $this->config();

		$result = array();

		$prefix = 'update';
		
		$data = array();

		if($this->id > 0)
		{
			# 新增权限
			if(Dever::load('manage/auth.oper', 3) == false)
			{
				return '';
			}
		
			$data = $this->load();

			if($data)
			{
				//$prefix = 'set';
			}
		}
		else
		{
			# 新增权限
			if(Dever::load('manage/auth.oper', 4) == false)
			{
				return '';
			}
		}
		
		//print_r($config['manage']['update']);die;
		$admin = Dever::load('manage/auth.info');
		$data['admin_id'] = $admin['id'];
		$data['role_id'] = $admin['role'];
		foreach($config['struct'] as $k => $v)
		{
			if(isset($v['update']))
			{
				//处理权限
				if(isset($v['auth']))
				{
					$auth = $this->value($v['auth'], $data);
					if(!$auth)
					{
						continue;
					}
				}
				# 处理show模式下的class
				$class = $this->update_class($v);
				
				$result[$k] = '<div class="form-group '.$class.'"><div class="margin-top">';
				
				$v['ext'] = $v['col'] = $k;
				
				$v['update'] = $v['update'] ? $v['update'] : 'text';
				
				if(isset($v['match']) && $v['match'] != 'option')
				{
					if(is_string($v['match']) && strpos($v['match'], '/') !== false)
					{
						$v['valid'] = 'validate[required,custom['.$k.']]';
					}
					else
					{
						$v['valid'] = 'validate[required]';
					}
				}
				
				if(is_string($v['update']) && strpos($v['update'], 'copy.') !== false)
				{
					$v['value'] = $v['update'];
					$v['update'] = 'hidden';
				}
				
				$this->update_html($v, $k, $k, $data, $result);

				$result[$k] .= '</div></div>';
			}
			elseif(isset($v['class']))
			{
				# 分割线
				$v['name'] = $this->value($v['name'], array('test' => 1));
				$result[$k] = Html::hr($v);
			}
		}
		
		if(Dever::input('ajax') == 1)
		{
			return implode('', $result);
		}

		//print_r($result);die;

		return $result;
	}
	
	/**
     * update_class
     * 
     * @return string
     */
	public function update_class($param, $key = 'show')
	{
		$class = '';
		if(isset($param[$key]))
		{
			if(is_array($param[$key]))
			{
				$parent = array();
				$child = array();
				foreach($param[$key] as $k => $v)
				{
					if(strpos($v, '_') !== false)
					{
						$temp = explode('_', $v);
						$parent[$temp[0]] = $key . '_' . $temp[0];
						$child[$v] = $key . '_' . $v;
					}
					elseif($v == 'no')
					{
						$child[$v] = $key . '_no';
					}
				}

				$class = implode(' ', $parent) . ' ' . implode(' ', $child);
			}
			else
			{
				$class = $key . '_input';
			}
		}
		
		return $class;
	}
	
	/**
     * update_html
     * 
     * @return string
     */
	public function update_html($param, $key, $index, $data, &$result)
	{
		if(isset($param['name']) && $param['update'] != 'hidden' && is_string($param['update']))
		{
			$result[$key] .= Html::pageLeft($param);
		}
		
		if($param['update'])
		{
			# 名称带前缀
			$param['name'] = 'update_' . $param['ext'];
			$param['index'] = $index;
			
			//$param['value'] = '';
			
			# 得到当前的值
			if(isset($data[$index]) && !isset($param['value']))
			{
				$param['value'] = $data[$index];
			}
			
			# 得到下级的值
			if(isset($param['child_value']))
			{
				$param['child_value'] = $this->value($param['child_value'], $data);
			}

			# 验证option是否是匿名函数
			if(isset($param['option']))
			{
				$param['option'] = $this->option($param['option']);
			}

			if(isset($data[$index]) && isset($param['key']) && $param['update'] != 'editor')
			{
				$param['value'] = Dever::pic($data[$index]);
			}
			
			# 处理下级字段
			if(isset($param['child']))
			{
				$param['child'] = $this->option($param['child']);
				if(isset($data[$key . '_parent']))
				{
					$param['parent_value'] = $data[$key . '_parent'];
				}
			}
			
			# 设置默认值
			if(isset($param['default']))
			{
				$param['default'] = $this->option($param['default']);
			}

			# 处理同步
			if(isset($param['sync']) && isset($data[$param['sync'][0]]) && strpos($key, '-'))
			{
				$temp = explode('-', $key);
				$load = Dever::load($temp[0] . '/' . $temp[1] . '-check', array('option_' . $param['sync'][1] => $data[$param['sync'][0]]));
				$param['value'] = isset($load[$temp[2]]) ? $load[$temp[2]] : $param['default'];
			}
			
			$method = $param['update'];
			
			if(is_array($method))
			{
				$result[$key] .= '<ul class="nav nav-tabs">{tab}<li><a href="javascript:;" class="dever_form_add" toggle_key="'.$key.'">增加一条</a></li></ul>';

				$tab = '';
				
				$this->update_child(array(), $method, $key, $result, -1);
				
				if(isset($param['value']) && $param['value'])
				{
					$param['value'] = json_decode(base64_decode($param['value']), true);
					
					foreach($param['value'] as $k => $v)
					{
						$this->update_child($v, $method, $key, $result, $k);

						$active = '';
						if($k == 0)
						{
							$active = 'active';
						}
						$k = $k+1;
						$tab .= '<li class="'.$active.'"><a href="javascript:;" id="tab-'.$key.'-child-'.$k.'" toggle_key="'.$key.'" toggle_id="'.$k.'" toggle_child=".dever_'.$key.'_child" toggle="#'.$key.'-child-'.$k.'" onclick="showToggle($(this))">第'.$k.'条</a></li>';
					}
				}
				else
				{
					$this->update_child(array(), $method, $key, $result, 0);

					$tab = '<li class="active"><a href="javascript:;" id="tab-'.$key.'-child-1" toggle_key="'.$key.'" toggle_id="1" toggle_child=".dever_'.$param['col'].'_child" toggle="#'.$key.'-child-1" onclick="showToggle($(this))">第1条</a></li>';
				}

				$result[$key] = str_replace('{tab}', $tab, $result[$key]);
			}
			else
			{
				if(isset($param['update_input']) && isset($data[$index . '_input']))
				{
					$param['update_input_data'] = explode(',', $data[$index . '_input']);
				}
				$result[$key] .= Html::$method($param);
			}

			if(isset($param['desc']) && ($param['update'] == 'text' || $param['update'] == 'password'))
			{
				$result[$key] .= Html::desc($param['desc']);
			}
		}
	}
	
	/**
     * update_action
     * 
     * @return array
     */
	private function update_child($data, $param, $key, &$result, $index)
	{
		$style = '';
		if($index != 0)
		{
			$style = 'style="display:none;"';
		}
		$result[$key] .= '<div id="'.$key.'-child-'.($index+1).'" class="dever_'.$key.'_child dever_form_add_child" '.$style.'><div class="dever_form_delete">[删除]</div>';
		foreach($param as $k => $v)
		{
			$result[$key] .= '<div class="form-group">';
			
			$v['type'] = $v['update'];

			if($index < 0)
			{
				$v['ext'] = $key . '_c_' . $v['col'];
			}
			else
			{
				$v['ext'] = $key . '_c_' . $index . '_i_' . $v['col'];
			}

			$this->update_html($v, $key, $v['col'], $data, $result);
			
			$result[$key] .= '</div>';
		}
		
		$result[$key] .= '</div>';
	}

	/**
     * update_action
     * 
     * @return array
     */
	public function update_action()
	{
		$method = false;

		$config = $this->config();

		# 验证头部权限
		$common = array();
		if(isset($config['top']) && is_string($config['top']))
		{
			$top = Dever::load('manage/auth.getTop', array($config['top']));
			if($top)
			{
				//Dever::setInput('update_' . $config['top_key'], $top['value']);
				$common[$config['top_key']] = $top['value'];
			}
		}
		
		$input = Dever::preInput('update_');

		if($this->id > 0)
		{
			# 新增权限
			if(Dever::load('manage/auth.oper', 3) == false)
			{
				return Dever::out('no');
			}
			
			Dever::setInput('where_id', $this->id);
			Dever::setInput('update_where_id', $this->id);
			
			$info = $this->load();

			if($info)
			{
				$method = 'update';

				$col	= Dever::input('col');
				$value	= Dever::input('value');

				if($col && $value)
				{
					$method = 'update_' . $col;
					Dever::setInput('set_' . $col, $value);
					Dever::setInput('update_' . $col, $value);

					$this->load($method);

					$struct = $this->struct_data($config['struct'], array($col => $value));
					Log::add($this->project, $this->table, array('method' => $method, 'input' => $struct, 'struct' => $config['struct']), $this->id);

					Dever::out('yes');

					return;
				}
				
				$prefix = 'set';
			}
		}
		if(!$method)
		{
			# 新增权限
			if(Dever::load('manage/auth.oper', 4) == false)
			{
				return Dever::out('no');
			}
			
			$method = 'insert';
			$prefix = 'add';
		}
		if($method && $input)
		{
			$update = array();
			$i = 0;
			$c = array();
			foreach($input as $k => $v)
			{
				if($v && is_string($v) && strpos($v, 'copy.') !== false)
				{
					$temp = str_replace('copy.', '', $v);
					if(isset($input['update_' . $temp]))
					{
						$v = $input['update_' . $temp];
					}
				}
				if(strpos($k, '_c_') !== false)
				{
					if($v)
					{
						$temp = explode('_c_', $k);
						$k = str_replace('update_', $prefix . '_', $temp[0]);
						if(!isset($c[$k]))
						{
							$c[$k] = array();
						}
						if(strpos($temp[1], '_i_') !== false)
						{
							$temp = explode('_i_', $temp[1]);
							
							if(!isset($c[$k][$temp[1]]))
							{
								$c[$k][$temp[1]] = 0;
							}
							else
							{
								$c[$k][$temp[1]]++;
							}
							
							$num = $c[$k][$temp[1]];
							
							$update[$i][$k][$num][$temp[1]] = $v;
						}
						else
						{
							//$update[$i][$k][$c][$temp[1]] = $v;
						}
					}
				}
				elseif(strpos($k, '__') !== false)
				{
					$temp1 = explode('__', $k);
					$temp2 = explode('_', $temp1[0]);
					$i = $temp2[1]-1;
					
					if(strpos($k, 'where_id') !== false)
					{
						//$update[$i]['where_id'] = $v;
					}
					else
					{
						$update[$i]['add_' . $temp1[1]] = $v;
					}
				}
				else
				{
					$ki = str_replace('update_', $prefix . '_', $k);
					
					if(strpos($ki, 'where_id') !== false)
					{
						if($method != 'insert')
						{
							$update[$i]['where_id'] = $v;
						}
					}
					else
					{
						if(strpos($k, '-'))
						{
							$k = str_replace('update_', '', $k);
							if(isset($config['struct'][$k]) && isset($config['struct'][$k]['sync']))
							{
								$s_col = $config['struct'][$k]['sync'][0];
								$m_col = $config['struct'][$k]['sync'][1];
								if($s_col == 'id')
								{
									$s_value = $this->id;
								}
								else
								{
									$s_one = $this->load('one', $this->id);
									$s_value = $s_one[$s_col];
								}
								$temp = explode('-', $k);
								$index = $temp[0] . '/' . $temp[1];

								$update_other_set[$index]['set_' . $temp[2]] = $v;
								$update_other_set[$index]['set_' . $m_col] = $s_value;

								$update_other_add[$index]['add_' . $temp[2]] = $v;
								$update_other_add[$index]['add_' . $m_col] = $s_value;

								$update_other_id[$index]['option_' . $m_col] = $s_value;
							}
						}
						else
						{
							$update[$i][$ki] = $v;
						}
					}
				}
				
				if($common)
				{
					foreach($common as $ki => $vi)
					{
						$update[$i][$prefix . '_' . $ki] = $vi;
					}
				}
			}

			//print_r($input);
			//print_r($update_other);die;
			//print_r($config['struct']);die;
			# 增加复制的数量
			$copy_num = Dever::input('copy_num');
			if($copy_num > 0)
			{
				$update[0]['where_id'] = 0;
				for($i = 1; $i < $copy_num; $i++)
				{
					$update[$i] = $update[0];
					# 缓存处理
					$update[$i]['rank'] = $i;
				}
			}

			//print_r($config);die;
			
			foreach($update as $k => $v)
			{
				$method = 'insert';
				$old = array();
				if(isset($v['where_id']) && $v['where_id'] > 0)
				{
					$method = 'update';
					$old = $this->load('one', $v['where_id']);
				}
				$id = $this->load($method, $v);
				
				if($id)
				{
					$struct = $this->struct_data($config['struct'], $v, $old);
					Log::add($this->project, $this->table, array('method' => $method, 'input' => $struct), $this->id > 0 ? $this->id : $id);
				}

				if(isset($update_other_id) && $update_other_id)
				{
					foreach($update_other_set as $k => $v)
					{
						$check = Dever::load($k . '-check', $update_other_id[$k]);
						//print_r($check);die;
						if($check)
						{
							$v['where_id'] = $check['id'];
							Dever::load($k . '-update', $v);
						}
						else
						{
							Dever::load($k . '-insert', $update_other_add[$k]);
						}
					}
				}
			}
		}

		Dever::out('yes');
	}

	private function struct_data($struct, $data, $old = array())
	{
		$log = $tlog = array();
		foreach($struct as $i => $j)
		{
			if(isset($j['list_name']))
			{
				$j['name'] = $j['list_name'];
			}
			elseif(strpos($j['name'], '-'))
			{
				$temp = explode('-', $j['name']);
				$j['name'] = $temp[0];
			}

			if(isset($j['list_table']))
			{
				$j['list'] = $j['list_table'];
				if($j['list'] == false)
				{
					$data[$i] = false;
				}
			}

			$list = (isset($j['list']) && is_string($j['list']) && $j['list'] != 'table');

			if(isset($data['where_' . $i]) && $data['where_' . $i])
			{
				$log[$j['name']] = $data['where_' . $i];
			}
			elseif(isset($data['set_' . $i]) && $data['set_' . $i])
			{
				$log[$j['name']] = $data['set_' . $i];
			}
			elseif(isset($data['add_' . $i]) && $data['add_' . $i])
			{
				$log[$j['name']] = $data['add_' . $i];
			}
			elseif(isset($data[$i]) && $data[$i])
			{
				$log[$j['name']] = $list ? $this->value($j['list'], $data) : $data[$i];
			}

			if(isset($log[$j['name']]))
			{
				$tlog[$j['name']] = $log[$j['name']];
				if(is_array($tlog[$j['name']]))
				{
					$tlog[$j['name']] = implode(',', $tlog[$j['name']]);
				}
				elseif(strpos($log[$j['name']], 'http://wx.qlogo.cn/') !== false || strpos($log[$j['name']], '.jpg') !== false || strpos($log[$j['name']], '.gif') !== false  || strpos($log[$j['name']], '.png') !== false)
				{
					$log[$j['name']] = '<a href="'.$log[$j['name']].'" target="_blank"><img src="'.$log[$j['name']].'" width="150"></a>';
				}
				elseif(is_numeric($log[$j['name']]) && (strpos($i, 'time') !== false || strpos($i, 'date') !== false))
				{
					$log[$j['name']] = date("Y-m-d H:i:s", $log[$j['name']]);
				}
				elseif(isset($j['decode']))
				{
					$log[$j['name']] = json_decode(base64_decode($log[$j['name']]), true);
				}

				if(isset($j['option']) && !$list)
				{
					$name = array();
					$option = $this->option($j['option']);

					foreach($option as $a => $b)
					{
						if(!is_array($log[$j['name']]))
						{
							if($a == $log[$j['name']])
							{
								$name[] = (is_array($b) && isset($b['name'])) ? $b['name'] : $b;
							}
						}
						elseif(in_array($a, $log[$j['name']]))
						{
							$name[] = (is_array($b) && isset($b['name'])) ? $b['name'] : $b;
						}
					}

					$log[$j['name']] = implode(',', $name);
				}

				if(isset($old[$i]) && $old[$i] != $tlog[$j['name']])
				{
					$log[$j['name']] = array('old' => $old[$i], 'new' => $log[$j['name']]);
				}
			}
		}

		return $log;
	}

	/**
     * delete_action
     * 
     * @return array
     */
	public function delete_action()
	{
		# 新增权限
		if(Dever::load('manage/auth.oper', 5) == false)
		{
			return Dever::out('no');
		}
			
		$this->load('delete');

		//$this->url(1)

		Dever::out('yes');
	}
	
	/**
     * stat 为主动统计提供相关数据
     * 
     * @return array
     */
	public function stat()
	{
		//bundle 包依赖工具
		$config = $this->config();

		if(empty($config['manage']['stat']))
		{
			Dever::abert('该项目主动统计功能未开放');
		}
		
		$data = $config['manage'];
		
		$data['table'] = '';
		
		$method = $this->project . '/' . $this->table . '-all';

		$gp = Dever::input('gp', 'month,day');

		$col = is_string($gp) ? explode(',', $gp) : $gp;

		$data['search'] = $this->stat_search($config['struct'], $gp, $config['manage']['stat'], $config['manage']['search']);

		foreach($col as $k => $v)
		{
			$this->stat_data($data, $config['struct'], $v, $method);
		}
		
		return $data;
	}
	
	/**
     * stat 为主动统计提供导出数据
     * 
     * @return array
     */
	public function stat_excel()
	{
		Dever::$global['base']['excel'] = 1;
		//print_r($data['thead']);die;
		$data = $this->stat();
		if($data && isset($data['list']))
		{
			$xls = new \PHPExcel();

			/*
			if(class_exists('\\PHPExcel'))
			{
				$xls = new \PHPExcel();
			}
			else
			{
				Dever::debug('Class \'PHPExcel\' not found');
				die;
			}
			*/
		
			$xls->getProperties()->setCreator("Maze Stat Excel")
							 ->setLastModifiedBy("Maze Stat Excel")
							 ->setTitle("Office 2007 XLSX Maze Document")
							 ->setSubject("Office 2007 XLSX Maze Document")
							 ->setDescription("Mazephp Stat Excel")
							 ->setKeywords("Mazephp Stat Excel")
							 ->setCategory("Mazephp Stat Excel");
			
			$info = $this->info();
			
			$name =  $info['list_header'] . '的' . $info['stat_desc'] . '-' . $info['project_name'];

			$start = 65;

			$s = 0;
			foreach($data['thead'] as $k => $v)
			{
				if($s > 0)
				{
					$xls->createSheet();
				}
				
				$act = $xls->setActiveSheetIndex($s);
				$act->setTitle($v[0]);
				
				$s++;
				$i = 0;
				foreach($v as $kj => $vj)
				{
					$num = 1;
					$cell = $this->_getChr($start, $i);
					$act->setCellValue($cell . '' . $num, $vj);
					$act->getColumnDimension($cell)->setWidth(20);
					$i++;
					
					if(isset($data['list'][$k]))
					{
						foreach($data['list'][$k] as $ki => $vi)
						{
							if(isset($vi[$kj]))
							{
								$num += 1;
								$act->setCellValue($cell . $num, $vi[$kj]);
							}
						}
					}
				}
			}
			
			//$act->getStyle('A1:'.$k.'20')->getAlignment()->setVertical(PHPExcel_Style_Alignment::VERTICAL_CENTER);   // 垂直居中
			//$act->getStyle('A1:'.$k.'20')->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);// 水平居中
			$xls->setActiveSheetIndex(0);
	    
			$filename = $name . '.xlsx';
			header('Pragma: public');
			header('Expires: 0');
			header('Cache-Control:must-revalidate,post-check=0,pre-check=0');
			header('Content-Type:application/force-download');
			header('Content-Type:application/vnd.ms-execl');
			header('Content-Type:application/octet-stream');
			header('Content-Type:application/download');
			header("Content-Disposition:attachment;filename='".$filename."'");
			header('Content-Transfer-Encoding:binary');

			$write = \PHPExcel_IOFactory::createWriter($xls, 'Excel2007');
			$write->save('php://output');
		}
	}

	/**
     * stat_search
     * 
     * @return mixed
     */
	private function stat_search($struct, $col, $stat, $search)
	{
		unset($struct['id']);
		unset($struct['cdate']);

		if($stat)
		{
			$stat = explode(',', $stat);
			foreach($stat as $k => $v)
			{
				if(isset($struct[$v]))
				{
					unset($struct[$v]);
				}
			}
		}

		$struct['day']['name'] = '天';
		$struct['month']['name'] = '月';
		$struct['year']['name'] = '年';
		$param['update'] = 'checkbox';
		$param['name'] = 'gp';
		$param['value'] = $col;
		$param['option'] = $struct;

		$html = '<form action="'.$this->url(7).'" method="post" id="f1">';
		$html .= '<div class="form-group">';
		$html .= Html::pageLeft(array('name' => '统计项', 'col' => 'col'));
		$html .= Html::radio($param);
		$html .= '</div>';

		$input = Dever::preInput('search_');

		if(isset($search['date']))
		{
			foreach($search['date'] as $k => $v)
			{
				$html .= '<div class="form-group">';
				$this->list_search_time($html, $input, 'search_', $k, $v, 'date');
				$html .= '</div>';
			}
			
			$input = Dever::preInput('search_');
			
			if($input)
			{
				foreach($input as $k => $v)
				{
					if($v && strpos($k, 'option_') !== false)
					{
						Dever::setInput(str_replace('search_', '', $k), $v);
					}
				}
			}
		}
			
		if(isset($search['fulltext']))
		{
			$html .= '<div class="form-group">';
			$this->list_search_select($html, $input, 'search_fulltext', '选择检索项', $search['fulltext']);
			
			$this->list_search_select($html, $input, 'search_fulltext_type', '', array
			(
				1 => '精确匹配',
				2 => '模糊匹配',
				3 => '大于',
				4 => '大于等于',
				5 => '小于',
				6 => '小于等于',
			), 1);

			$value = array();
			$value['name'] = 'search_fulltext_value';
			$value['class'] = 'form-control';
			if(isset($input[$value['name']]))
			{
				$value['value'] = $input[$value['name']];
				
				$fix = '';
				if($input['search_fulltext_type'] == 2)
				{
					$fix = '%';
				}

				Dever::setInput('option_' . $input['search_fulltext'], $fix . $value['value'] . $fix);

				//print_r(Dever::input('option_name'));die;
			}
			$html .= Html::text($value, false);
			$html .= '</div>';
		}

		$html .= '<input type="hidden" name="excel" id="excel"  value="2"><a href="javascript:;" class="btn btn-primary" onclick="$(\'#f1\').attr(\'action\', \''.$this->url(7).'\').submit();">确定</a>';
		
		$html .= '&nbsp;&nbsp;<a href="javascript:;" class="btn btn-warning" onclick="$(\'#f1\').attr(\'action\', \''.$this->url(9).'\').submit();">下载报表</a>';

		$html .= '</form>';

		

		return $html;
	}

	/**
     * stat_data
     * 
     * @return mixed
     */
	private function stat_data(&$data, $struct, $col, $method)
	{
		Dever::setInput('group', $col);
		switch($col)
		{
			case 'year':
				$name = '按年统计';
				break;
			case 'month':
				$name = '按月统计';
				break;
			case 'day':
				$name = '按天统计';
				break;
			default:
				$name = $struct[$col]['name'];
				break;
		}
		$list = Dever::load($method, array('group' => $col, 'col' => 'count(1) as total,id'));
		
		$data['table'] .= '<div style="overflow-y:auto;width:100%;max-height:600px;"><table class="table table-bordered table-striped">';
		
		$data['table'] .= '<thead><tr><th>'.$name.'</th><th>统计数字</th></tr></thead>';
		
		$data['thead'][$col] = array($name, '统计数字'); 

		if($list)
		{
			$data['table'] .= '<tbody>';
			$total = 0;
			foreach($list as $k => $v)
			{
				if(isset($v[$col]))
				{
					if(isset($struct[$col]['option']))
					{
						$option = $this->option($struct[$col]['option']);
						if(isset($option[$v[$col]]))
						{
							$v[$col] = $option[$v[$col]];
							if(is_array($v[$col]))
							{
								$v[$col] = $v[$col]['name'];
							}
						}
					}

					$data['table'] .= '<tr><td width="60%" style="word-break:break-all;word-wrap:break-word;">'.$v[$col].'</td><td>'.$v['total'].'</td></tr>';
					$total += $v['total'];
					
					$data['list'][$col][] = array($v[$col], $v['total']);
				}
				
			}

			$data['table'] .= '</tbody>';

			if($total > 0 && $k > 0)
			{
				$data['table'] .= '<tr><td style="font-weight:bold">汇总</td><td style="font-weight:bold">'.$total.'</td></tr>';
				$data['list'][$col][] = array('汇总', $total);
			}

		}
		
		$data['table'] .= '</table></div>';
	}


	/**
     * load
     * 
     * @return mixed
     */
	private function load($method = 'one', $param = array())
	{
		return Dever::load($this->project . '/' . $this->table . '-' . $method, $param);
	}
	
	/**
     * __call
     *
     * @return object
     */
    public function __call($method, $param)
    {
    	if(strpos($method, 'html_') !== false)
    	{
    		return Html::text($param);
    	}

    	return $this;
    }

    /**
     * option
     * 
     * @return string
     */
    private function option($option)
    {
    	if(is_object($option))
    	{
    		$function = $option;
			$option = $function();
    	}

    	return $option;	
    }
    
    /**
     * option
     * 
     * @return string
     */
    public function manage()
    {
    	$config = $this->config();

    	return $config;	
    }
    
    /**
     * valid
     * 
     * @return string
     */
    public function valid()
    {
		//http://placehold.it/350x150
    	$config = $this->config();
    	if(isset($config['struct']))
    	{
			$rule = array();
			$rule['required']['regex'] = 'none';
			$rule['required']['alertText'] = '此项不能为空';
			$rule['required']['alertTextCheckboxMultiple'] = '请选择一个选项';
			$rule['required']['alertTextCheckboxe'] = '请选择一个选项';
			$rule['required']['alertTextDateRange'] = '日期范围不可空白';

			foreach($config['struct'] as $k => $v)
			{
				if(isset($v['match']) && $v['match'] != 'option' && is_string($v['match']) && strpos($v['match'], '/') !== false)
				{
					$rule[$k]['regex'] = $v['match'];
					$rule[$k]['alertText'] = $v['desc'];
				}
			}
		}
		
		$script = '<script>';
		
        
		$script .= '(function($){
		$.fn.validationEngineLanguage = function(){
		};
		$.validationEngineLanguage = {
			newLang: function(){
				$.validationEngineLanguage.allRules = ';
		//此处进行了转义过滤，其实也可以使用json_encode的第二个参数，不过只支持5.4以上版本，考虑到虚拟主机就放弃了
		$script .= str_replace(array('\\\\', '\\/', '"/', '/"'), array('\\', '/', '/', '/'), json_encode($rule)) . ';';
		$script .= '}
			};
			$.validationEngineLanguage.newLang();
		})(jQuery);';
		$script .= '$(document).ready(function(){
        $(".form1").validationEngine("attach",{onSuccess:function(){submit();}});
        });';
		$script .= '</script>';
		
		//print_r($script);die;

    	return $script;	
    }
    
    /**
     * save
     * 
     * @return string
     */
    public function save()
    {
		$config = $this->config();
		
		if(isset($config['manage']['save']))
		{
			return 'yes';
		}
		
		return 'no';
	}
}
